/*
 *  Copyright (C) 2007  Thadeu Lima de Souza Cascardo <cascardo@holoscopio.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */


#include <glib.h>
#include <atompub/atom.h>
#include <atompub/atom-glib.h>

static gchar *
giochannel_iri_to_filename (AtomCtx *ctx, IRI *iri)
{
  gchar *root = atom_config_get_str (ctx, "giochannel", "root");
  gchar *path = iri_get_path (iri);
  gchar *filename = g_build_filename (root, path, NULL);
  g_free (root);
  return filename;
}

static AtomEntry *
giochannel_atom_retrieve_resource (AtomCtx *ctx, IRI *iri)
{
  gchar *filename;
  GIOChannel *channel;
  GError *error = NULL;
  gchar *data;
  gsize len;
  AtomEntry *atom;
  filename = giochannel_iri_to_filename (ctx, iri);
  channel = g_io_channel_new_file ((const gchar *) filename, "r", &error);
  g_free (filename);
  if (channel == NULL)
    {
      AtomError *aerr = atom_error_new_from_gerror (error);
      atom_error_set (ctx, aerr);
      g_error_free (error);
      return NULL;
    }
  error = NULL;
  if (g_io_channel_read_to_end (channel, &data, &len, &error) !=
      G_IO_STATUS_NORMAL)
    {
      AtomError *aerr = atom_error_new_from_gerror (error);
      g_io_channel_unref (channel);
      atom_error_set (ctx, aerr);
      g_error_free (error);
      return NULL;
    }
  g_io_channel_unref (channel);
  atom = atom_entry_new_data_len (data, len);
  g_free (data);
  return atom;
}

AtomBackend *
giochannel_backend (void)
{
  AtomBackend *backend;
  backend = atom_backend_new ();
  atom_backend_retrieve_resource_set (backend,
				      giochannel_atom_retrieve_resource);
  return backend;
}
